<?php
/**
 * @copyright (c) JTL-Software-GmbH
 * @license http://jtl-url.de/jtlshoplicense
 * modified by Conny Henn 01/2020
 */

/** @global Plugin $oPlugin */
// Prüfen ob Plugin aktiviert ist
if (isset($oPlugin->oPluginEinstellungAssoc_arr['hb_nutzen']) &&
    $oPlugin->oPluginEinstellungAssoc_arr['hb_nutzen'] === 'Y' &&
    strlen($oPlugin->oPluginEinstellungAssoc_arr['hb_token']) > 0 )
{
    define('HB_URL', 'https://www.hb-intern.de/www/hbm/api/live_rechtstexte.htm');

    $nCaching = (int)$oPlugin->oPluginEinstellungAssoc_arr['hb_intervall']; // Hours
    if ($nCaching < 1) {
        $nCaching = 24;
    }
    checkHaendlerbund($oPlugin, $nCaching);
}

/**
 * @param Plugin $oPlugin
 * @param string $cType
 * @param string $cFormat
 * @param int    $kUpdate
 * @param int    $kSprache
 * @return bool
 */
function getNewHaendlerbund($oPlugin, $cType, $cFormat, $kUpdate, $kSprache)
{
    Jtllog::writeLog(
        'Update: kUpdate (haendlerbund) ' . $kUpdate . ' - cType ' .
            $cType . ' - kSprache ' . $kSprache . ' wird geladen ...',
        JTLLOG_LEVEL_DEBUG,
        false,
        'kPlugin',
        $oPlugin->kPlugin
    );
		
	// passende ID zuweisen
	/** Document Id's
	"12766C46A8A":"AGBs",
	"12766C53647":"Widerrufsbelehrung",
	"1452C24576D":"Widerrufsbelehrung digitale Waren",
	"1463C5DBF05":"Widerrufsbelehrung Dienstleistungen",
	"12766C58F26":"Zahlung und Versand",
	"160DEDA9674":"Datenschutzerklärung", oder 12766C5E204
	"1293C20B491":"Impressum",
	"134CBB4D101":"Batteriehinweise"
	**/
	switch ($cType) {
	case 'AGB':
		$cId_Type = '12766C46A8A';
		break;
	case 'WRB':
		switch ($oPlugin->oPluginEinstellungAssoc_arr['hb_akt_wrb']) {
		// normale WRB
		case 'WRB1':
			$cId_Type = '12766C53647';
			break;

		// digital WRB
		case 'WRB2':
			$cId_Type = '1452C24576D';
			break;
		
		// digital Dienstleistung
		case 'WRB3':
			$cId_Type = '1463C5DBF05';
			break;
		}
		break;
	case 'IMPRESSUM':
		$cId_Type = '1293C20B491';
		break;
	case 'DATENSCHUTZ':
		$cId_Type = '12766C5E204';
		break;
	case 'BATTERIE':
		$cId_Type = '134CBB4D101';
		break;
	case 'ZAHLUNG_VERSAND':
		$cId_Type = '12766C58F26';
		break;
	}

	// Parameter für URL zusammenfügen
    $cParams  = http_build_query([
        'APIkey'      => '1IqJF0ap6GdDNF7HKzhFyciibdml8t4v',
        'AccessToken' => $oPlugin->oPluginEinstellungAssoc_arr['hb_token'],
        'mode'        => $cFormat === 'html' ? 'classes' : 'plain',
        'did'         => $cId_Type,
		'lang'		  => $kSprache
    ], '', '&');
    $cURL     = HB_URL . '?' . $cParams;

	/** 
	 * abholen der Inhalte beim Händlerbund
	*/
	// $cContent = @http_get_contents($cURL);			// original von JTL
	
	// alternative Funktion über CURL
	// function get_content($URL){
    //  $ch = curl_init();
    //  curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    //  curl_setopt($ch, CURLOPT_URL, $URL);
    //  $data = curl_exec($ch);
    //  curl_close($ch);
    //  return $data;
	// }
	// $cContent = get_content($cURL);

	// oder
	$cContent = file_get_contents($cURL);				// alternative - hat weniger Abbrüche - allow_url_fopen muss aktiv sein
	
	
    $cContent = utf8_decode($cContent);
	
	// Fehlerprüfung
    if (in_array($cContent, ['DOCUMENT_NOT_AVAILABLE', 'SHOP_NOT_FOUND', 'WRONG_API_KEY', 'LANG_NOT_AVAILABLE'], true)) {
        Jtllog::writeLog('Fehler (haendlerbund): ' . $cContent, JTLLOG_LEVEL_ERROR, false, 'kPlugin', $oPlugin->kPlugin);
        $cContent = false;
    }
    // Wurde ein Text uebermittelt?
    if (empty($cContent)) {
        Jtllog::writeLog(
            "Fehler (haendlerbund): " . "Kein Inhalt fuer {$cType}, {$cFormat} vorhanden.",
            JTLLOG_LEVEL_ERROR,
            false,
            'kPlugin',
            $oPlugin->kPlugin
        );
        $cContent = false;
    }
	
	// wenn Sprache nicht definiert dann ist sie deutsch
	if (empty($kSprache)){
		$kSprache = 'de';
	}
	
	// aus Sprache ISO -> ID machen | Suche in gibAlleSprachen nach ISO und gib ID zurück
	$oSprache_arr = gibAlleSprachen();
	$key = array_search($kSprache, array_column($oSprache_arr, 'cISO639'));
	$kSprache = $oSprache_arr[$key]->kSprache;

    if ($cContent !== false) {
        $cContent = Shop::DB()->escape($cContent);
        $cSpalte  = '';
        $cTabelle = '';
        $cWhere   = '';
        switch ($cType) {
            // AGB
            case 'AGB':
				$cSpalte  = "cAGBContentHtml";
				$cTabelle = "ttext";
				$cWhere   = " kSprache = " . (int)$kSprache;
				if ($cFormat === 'txt') {
					// $cContent = strip_tags($cContent);
					$cSpalte = "cAGBContentText";
				}
                break;

            // WRB
            case 'WRB':
				$cSpalte  = "cWRBContentHtml";
				$cTabelle = "ttext";
				$cWhere   = " kSprache = " . (int)$kSprache;
				if ($cFormat === 'txt') {
					// $cContent = strip_tags($cContent);
					$cSpalte = "cWRBContentText";
				}
                break;

            // Impressum
            case 'IMPRESSUM':
				$cSpalte  = "cContent";
				$cTabelle = "tlinksprache";
				$oLink    = Shop::DB()->query("SELECT kLink FROM tlink WHERE nLinkart = " . LINKTYP_IMPRESSUM, 1);
				$cWhere   = "cISOSprache = '" . gibSprachKeyISO("", $kSprache) . "' AND kLink = " . $oLink->kLink;
                break;

            // Datenschutz
            case 'DATENSCHUTZ':
				$cSpalte  = "cContent";
				$cTabelle = "tlinksprache";
				$oLink    = Shop::DB()->query("SELECT kLink FROM tlink WHERE nLinkart = " . LINKTYP_DATENSCHUTZ, 1);
				$cWhere   = "cISOSprache = '" . gibSprachKeyISO("", $kSprache) . "' AND kLink = " . $oLink->kLink;
                break;

            // Batteriegesetzhinweise
            case 'BATTERIE':
				$cSpalte  = "cContent";
				$cTabelle = "tlinksprache";
				$oLink    = Shop::DB()->query("SELECT kLink FROM tlink WHERE nLinkart = " . LINKTYP_BATTERIEGESETZ_HINWEISE, 1);
				$cWhere   = "cISOSprache = '" . gibSprachKeyISO("", $kSprache) . "' AND kLink = " . $oLink->kLink;
                break;
				
			// Zahlung und Versand
            case 'ZAHLUNG_VERSAND':
				$cSpalte  = "cContent";
				$cTabelle = "tlinksprache";
				$oLink    = Shop::DB()->query("SELECT kLink FROM tlink WHERE nLinkart = " . LINKTYP_VERSAND, 1);
				$cWhere   = "cISOSprache = '" . gibSprachKeyISO("", $kSprache) . "' AND kLink = " . $oLink->kLink;
                break;	
        }
		
		// Signatur schreiben
		if (isset($oPlugin->oPluginEinstellungAssoc_arr['hb_signature']) &&
				$oPlugin->oPluginEinstellungAssoc_arr['hb_signature'] === 'Y'){
			if ($cFormat === 'txt') {
				$cContent = $cContent."\n\rSignatur: ".date("dmyHi");
			} else {
				$cContent = $cContent."<br><small>Signatur: ".date("dmyHi")."</small><br>";
			}
		}
		
		// CSS schreiben
		if (isset($oPlugin->oPluginEinstellungAssoc_arr['hb_css_include']) &&
				$oPlugin->oPluginEinstellungAssoc_arr['hb_css_include'] === 'Y' && $cFormat <> 'txt'){
			if ( isset($oPlugin->oPluginEinstellungAssoc_arr['hb_css']) &&
				!empty($oPlugin->oPluginEinstellungAssoc_arr['hb_css']) && $cFormat <> 'txt' ){
				$cContent = "<style>".$oPlugin->oPluginEinstellungAssoc_arr['hb_css']."</style>".$cContent;
			}
		}
		
		// Datenbank updaten
        Shop::DB()->query("
                UPDATE " . $cTabelle . "
                    SET " . $cSpalte . " = '" . $cContent . "'
                    WHERE " . $cWhere, 3
        );
        Shop::DB()->query("
                UPDATE xplugin_jtl_haendlerbund_tupdate
                    SET dAktualisiert = now(), nVersuch = 0
                    WHERE kUpdate = " . (int)$kUpdate, 3
        );
        Jtllog::writeLog(
            '(haendlerbund) kUpdate: ' . $kUpdate . ' ' . $cType . ' wurde aktualisiert.',
            JTLLOG_LEVEL_DEBUG,
            false,
            'kPlugin',
            $oPlugin->kPlugin
        );
        
        return true;
    } else {
        Shop::DB()->query("
                UPDATE xplugin_jtl_haendlerbund_tupdate
                    SET nVersuch = nVersuch + 1
                    WHERE kUpdate = " . (int)$kUpdate, 3
        );
        Jtllog::writeLog(
            '(haendlerbund) kUpdate ' . $kUpdate . ' war nicht erfolgreich.',
            JTLLOG_LEVEL_ERROR,
            false,
            'kPlugin',
            $oPlugin->kPlugin
        );
    }
    
    return false;
}



/**
 * Prueft ob DB noch aktuell ist
 *
 * @param Plugin $oPlugin
 * @param int $nCaching
 */
function checkHaendlerbund($oPlugin, $nCaching)
{
	/**
	* DB Felder: 
	* kUpdate = fortlaufende Nummer
	* cType   = ID für den URL Aufruf = TextID
	* cFormat = txt /  html 
	* nAktiv  = ob nach zu vielen Fehlern deaktivert wurde
	* nVersuch = Menge der Versuche zum aktualisieren
	* dAktualisiert = letzte Aktualisierung        2020-01-22 10:33:05
	* dVersuch = letzter Versuch der Aktualiserung 2020-01-22 10:33:05
	* NEU
	* cSprache = Sprache [cISO639] de,en,fr,
	* cTextType= AGB, WRB, IMPRESSUM, DATENSCHUTZ, BATTERIE, ZAHLUNG_VERSAND
	*/

	//
	// Wenn Fehler beim letzten Aufruf, dann 24 Stunden pausieren
	//
	
	// Datensatz mit dem ältesten Datum suchen
    $oLatestTry = Shop::DB()->query("SELECT * FROM xplugin_jtl_haendlerbund_tupdate ORDER BY dVersuch DESC LIMIT 1", 1);
	
	// Verzögerung errechnen - Menge der Versuche mal 60 Sekunden
    $nDelay     = pow($oLatestTry->nVersuch, 2) * 60;
    if($nDelay > 86400) {
        $nDelay = 86400;  		// 24 Stunden
    }
    $time = strtotime($oLatestTry->dVersuch);
    if ($time + $nDelay > time()) {
        Jtllog::writeLog(
            '(haendlerbund) Aktualisierung pausiert aufgrund von Fehlern beim letzten Abruf. ' .
            'N&auml;chste Aktualisierung fr&uuml;hstens '.  date('d.m.Y H:i:s', $time + $nDelay),
            JTLLOG_LEVEL_DEBUG,
            false,
            'kPlugin',
            $oPlugin->kPlugin
        );
        return;
    }    
    
	// alle Datensätze zum aktualisieren auslesen
	$oUpdate_arr = Shop::DB()->query("
		SELECT *
			FROM xplugin_jtl_haendlerbund_tupdate
			WHERE (dAktualisiert = '0000-00-00 00:00:00'
				OR DATE_ADD(dAktualisiert, INTERVAL " . (int)$nCaching . " HOUR) < now())
				AND nAktiv = 1", 2
	);
	Jtllog::writeLog(
		'(haendlerbund) Text: ' . print_r($oUpdate_arr, true),
		JTLLOG_LEVEL_DEBUG,
		false,
		'kPlugin',
		$oPlugin->kPlugin
	);

	if (count($oUpdate_arr) > 0) {
		foreach ($oUpdate_arr as $oUpdate) {
			if (isset($oUpdate->kUpdate) && $oUpdate->kUpdate > 0 && $oUpdate->nOnOff == 1) {
				
				// bei zu vielen Fehlern Abgleich deaktiveren
				if (isset($oUpdate->nVersuch) && $oUpdate->nVersuch >= 999) {
					Shop::DB()->query("
						UPDATE xplugin_jtl_haendlerbund_tupdate
							SET nAktiv = 0
							WHERE kUpdate = " . (int)$oUpdate->kUpdate, 3
					);
					Jtllog::writeLog(
						'kUpdate (haendlerbund) ' . $oUpdate->kUpdate . ' wurde deaktiviert!',
						JTLLOG_LEVEL_ERROR,
						true,
						'kPlugin',
						$oPlugin->kPlugin
					);

					continue;
				}
				
				// $nDelay
				Shop::DB()->query("
					UPDATE xplugin_jtl_haendlerbund_tupdate 
						SET dVersuch = NOW() 
						WHERE kUpdate = " . (int)$oUpdate->kUpdate, 3
				);

				// Bug Korrektur - DE darf als Parameter nicht mit übergeben werden
				if ($oUpdate->cSprache == 'de'){
					$cSprache = '';
				} else {
					$cSprache = $oUpdate->cSprache;
				}
				
				// Updaten
				if (!getNewHaendlerbund(
					$oPlugin,
					$oUpdate->cTextType,
					$oUpdate->cFormat,
					$oUpdate->kUpdate,
					$cSprache
				)) {
					break;
				}
			}
		}
	}
}
